@@ -32,6 +32,7 @@ class Settings(DictParser):
     key_link_on_import = 'link_on_import'
     key_showdiff = 'showdiff'
     key_upignore = 'upignore'
+    key_impignore = 'impignore'
     key_cmpignore = 'cmpignore'
     key_instignore = 'instignore'
     key_workdir = 'workdir'
@@ -40,6 +41,7 @@ class Settings(DictParser):
     key_filter_file = 'filter_file'
     key_diff_command = 'diff_command'
     key_template_dotfile_default = 'template_dotfile_default'
+    key_ignore_missing_in_dotdrop = 'ignore_missing_in_dotdrop'
 
     # import keys
     key_import_actions = 'import_actions'
@@ -52,11 +54,12 @@ class Settings(DictParser):
                  import_variables=[], keepdot=False,
                  link_dotfile_default=LinkTypes.NOLINK,
                  link_on_import=LinkTypes.NOLINK, longkey=False,
-                 upignore=[], cmpignore=[], instignore=[],
+                 upignore=[], cmpignore=[], instignore=[], impignore=[],
                  workdir='~/.config/dotdrop', showdiff=False,
                  minversion=None, func_file=[], filter_file=[],
                  diff_command='diff -r -u {0} {1}',
-                 template_dotfile_default=True):
+                 template_dotfile_default=True,
+                 ignore_missing_in_dotdrop=False):
         self.backup = backup
         self.banner = banner
         self.create = create
@@ -72,6 +75,7 @@ class Settings(DictParser):
         self.upignore = upignore
         self.cmpignore = cmpignore
         self.instignore = instignore
+        self.impignore = impignore
         self.workdir = workdir
         if ENV_WORKDIR in os.environ:
             self.workdir = os.environ[ENV_WORKDIR]
@@ -82,6 +86,7 @@ class Settings(DictParser):
         self.filter_file = filter_file
         self.diff_command = diff_command
         self.template_dotfile_default = template_dotfile_default
+        self.ignore_missing_in_dotdrop = ignore_missing_in_dotdrop
 
     def _serialize_seq(self, name, dic):
         """serialize attribute 'name' into 'dic'"""
@@ -105,6 +110,7 @@ class Settings(DictParser):
             self.key_minversion: self.minversion,
             self.key_diff_command: self.diff_command,
             self.key_template_dotfile_default: self.template_dotfile_default,
+            self.key_ignore_missing_in_dotdrop: self.ignore_missing_in_dotdrop,
         }
         self._serialize_seq(self.key_default_actions, dic)
         self._serialize_seq(self.key_import_actions, dic)
@@ -113,6 +119,7 @@ class Settings(DictParser):
         self._serialize_seq(self.key_cmpignore, dic)
         self._serialize_seq(self.key_upignore, dic)
         self._serialize_seq(self.key_instignore, dic)
+        self._serialize_seq(self.key_impignore, dic)
         self._serialize_seq(self.key_func_file, dic)
         self._serialize_seq(self.key_filter_file, dic)
 
